/* Copyright (c) 1999 TTK Ciar / Bill Moyer
 * All rights reserved.
 *
 * This code is derived from code developed by TTK Ciar and Bill Moyer.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright 
 *     notice, this list of conditions and the following disclaimer.
 *  2. The name of the author may not be used to endorse or promote 
 *     products derived from this software without specific prior 
 *     written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

static int major_revision = 3;
static int minor_revision = 0;

static int   width      = 16;  /* number of bytes processed per line */
static int   hexcase    =  0;  /* 0 for lowercase hex, 1 for uppercase */
static int   useheader  =  1;  /* 1 to generate filename/filesize prefix line */

static int   pix = 0;
static char  paddedstr[8][128];
/* interface */ char *padstr ( char *str, char pad, int w_ ) {
  int len, r, i, w;
  if ( pix >= 8 ) pix = 0;
  paddedstr[pix][0] = 0;
  if ( str == NULL ) return paddedstr[pix++];
  w = w_;
  if ( w > 127 ) return paddedstr[pix++];
  len = strlen(str);
  if ( len >= w ) { 
    strcpy ( paddedstr[pix], str ); 
    return ( paddedstr[pix++] );
    }
  r = w - len;
  for ( i = 0; i < r; i++ ) paddedstr[pix][i] = pad;
  strcpy ( &(paddedstr[pix][r]), str );
  return ( paddedstr[pix++] );
  }

/* interface */ char *padnum ( int num, int w ) {
  char numstr[32];
  sprintf ( numstr, "%d", num );
  return padstr ( numstr, '0', 2 );
  }

/* interface */ char *padhex ( int num, int w, int ucase ) {
  char numstr[32];
  if ( ucase ) 
       sprintf ( numstr, "%X", num );
  else sprintf ( numstr, "%x", num );
  return padstr ( numstr, '0', 2 );
  }

/* interface */ int qd3 ( char *name, int k, char *buf ) {
   int  n, i, j, bit, quit, res;
   char *asciistr;
   int  idatasectionindex = 0;
   int  fsz;
   unsigned char foo;

   asciistr = (char*) malloc ( width + 1 );
   asciistr[width] = 0;
   if ( useheader ) printf ("[%s] %d\n",name,k);
   n = 0; i = 0; j = 0; quit = 0;
   while ( !quit ) {
     if ( n < k ) foo = buf[n++];
     else {
       res = 0;
       foo = 0;
       if ( i % width == 0 ) quit = 1;
       }
     if ( i % width == 0 ) if ( !quit )
       printf ( "%6u %5X : ", j, j );
     asciistr[i] = foo;
     if ( ( (unsigned char)asciistr[i] < ' ' ) ||
          ( (unsigned char)asciistr[i] > 126 )    )
        asciistr[i] = '.';
     if ( !quit || i ) {
        if ( res == 0 ) printf (" **");
        else printf ("%3s",padhex(foo,2,hexcase));
        }
     i++; j++;
     if ( i % width == 0 ) { 
       asciistr[i] = 0;
       i = 0; 
       printf (" |%s\n",asciistr); 
       }
     }
   free ( asciistr );
   return 0;
   }

/* interface */ int qd3ctl ( int code, int val ) {
  switch ( code ) {
    case 0 : if ( val > 0 ) width = val; else return -1; return 0;
    case 1 : hexcase = val; return 0;
    case 2 : useheader = val; return 0;
    case 3 : return width; 
    case 4 : return hexcase; 
    case 5 : return useheader; 
    case 6 : return major_revision; 
    case 7 : return minor_revision; 
    }
  return -2;
  }

